@@ -139,8 +139,8 @@ class TestTaggedPageTreeStore(tests.TestCase):
 
 		# Try some TreeView methods
 		path = Path('Test:foo')
-		self.assertTrue(treeview.select_page(path))
-		self.assertEqual(treeview.get_selected_path(), path)
+		self.assertTrue(treeview.set_current_page(path))
+		# TODO assert something
 		treepath = treeview.get_model().get_treepath(path)
 		self.assertTrue(not treepath is None)
 		col = treeview.get_column(0)
