@@ -457,7 +457,8 @@ class XDotAttrParser:
         return res
 
     def read_number(self):
-        return int(self.read_code())
+        return int(float(self.read_code()))
+			# Changed to float here to fix launchpag bug #1285396
 
     def read_float(self):
         return float(self.read_code())
